@@ -87,7 +87,8 @@
 #define NS_STYLE_USER_SELECT_TOGGLE     5
 #define NS_STYLE_USER_SELECT_TRI_STATE  6
 #define NS_STYLE_USER_SELECT_AUTO       7 // internal value - please use nsFrame::IsSelectable()
-#define NS_STYLE_USER_SELECT_MOZ_ALL    8 // force selection of all children - bug 48096
+#define NS_STYLE_USER_SELECT_MOZ_ALL    8 // force selection of all children, unless an ancestor has NONE set - bug 48096
+#define NS_STYLE_USER_SELECT_MOZ_NONE   9 // Like NONE, but doesn't change selection behavior for descendants whose user-select is not AUTO.
 
 // user-input
 #define NS_STYLE_USER_INPUT_NONE      0
@@ -548,14 +549,15 @@
 // Note: these values pickup after the text-align values because there
 // are a few html cases where an object can have both types of
 // alignment applied with a single attribute
-#define NS_STYLE_VERTICAL_ALIGN_BASELINE        10
-#define NS_STYLE_VERTICAL_ALIGN_SUB             11
-#define NS_STYLE_VERTICAL_ALIGN_SUPER           12
-#define NS_STYLE_VERTICAL_ALIGN_TOP             13
-#define NS_STYLE_VERTICAL_ALIGN_TEXT_TOP        14
-#define NS_STYLE_VERTICAL_ALIGN_MIDDLE          15
-#define NS_STYLE_VERTICAL_ALIGN_TEXT_BOTTOM     16
-#define NS_STYLE_VERTICAL_ALIGN_BOTTOM          17
+#define NS_STYLE_VERTICAL_ALIGN_BASELINE             10
+#define NS_STYLE_VERTICAL_ALIGN_SUB                  11
+#define NS_STYLE_VERTICAL_ALIGN_SUPER                12
+#define NS_STYLE_VERTICAL_ALIGN_TOP                  13
+#define NS_STYLE_VERTICAL_ALIGN_TEXT_TOP             14
+#define NS_STYLE_VERTICAL_ALIGN_MIDDLE               15
+#define NS_STYLE_VERTICAL_ALIGN_TEXT_BOTTOM          16
+#define NS_STYLE_VERTICAL_ALIGN_BOTTOM               17
+#define NS_STYLE_VERTICAL_ALIGN_MIDDLE_WITH_BASELINE 18
 
 // See nsStyleDisplay
 #define NS_STYLE_VISIBILITY_HIDDEN              0
