@@ -359,7 +359,8 @@ typedef PRUint32 nsReflowStatus;
 #define NS_FRAME_IS_TRUNCATED(status) \
   (0 != ((status) & NS_FRAME_TRUNCATED))
 #define NS_FRAME_SET_TRUNCATION(status, aReflowState, aMetrics) \
-  if (!aReflowState.mFlags.mIsTopOfPage &&                      \
+  if (aReflowState.availableHeight != NS_UNCONSTRAINEDSIZE &&   \
+      !aReflowState.mFlags.mIsTopOfPage &&                      \
       aReflowState.availableHeight < aMetrics.height)           \
     status |= NS_FRAME_TRUNCATED;                               \
   else                                                          \
